// $Id: ColorSelectionFrame.cpp,v 1.47 2013/02/22 12:05:49 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "ColorSelectionFrame.h"
#include "ColorSelectionFrame_p.h"

#include "ColorGrabObject.h"
#include "GridLayout.h"
#include "IconEngine.h"
#include "IconSize.h"
#include "Icons.h"
#include "InformationDialog.h"
#include "AnimatedLineEditor.h"
#include "Str.h"
#include "TransitionWidget.h"

#include <QLayout>
#include <QToolButton>
#include <QValidator>
#include <QPainter>

namespace PRIVATE
{

    //___________________________________________________________________________________
    ColorWidget::ColorWidget( QWidget* parent ):
        QPushButton( parent ),
        Counter( "ColorWidget" )
    {
        Debug::Throw( "ColorWidget::ColorWidget" );
        setIcon( IconEngine::get( ICONS::COLOR_PICKER ) );
        setText( "Pick a New Color" );

        ColorGrabObject* object = new ColorGrabObject( this );
        connect( object, SIGNAL( colorSelected( QString ) ), this, SIGNAL( colorSelected( QString ) ) );

    }

    //___________________________________________________________________________________
    ColorDisplay::ColorDisplay( QWidget* parent ):
        QFrame( parent ),
        Counter( "ColorDisplay" )
    {
        Debug::Throw( "ColorDisplay::ColorDisplay" );
        setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
        setFixedSize( IconSize( IconSize::Huge ) );
    }

    //___________________________________________________________________________________
    void ColorDisplay::setColor( const QColor& color )
    { color_ = color; }

    //___________________________________________________________________________________
    void ColorDisplay::paintEvent( QPaintEvent* event )
    {
        if( color_.isValid() )
        {
            QPainter painter( this );
            painter.setClipRegion( event->region() );
            painter.setBrush( color_ );
            painter.setPen( Qt::NoPen );

            const int frameWidth( this->frameWidth() );
            painter.drawRect( frameRect().adjusted( frameWidth, frameWidth, -frameWidth, -frameWidth )  );
            painter.end();
        }

        QFrame::paintEvent( event );

    }

}

//_________________________________________________________
ColorSelectionFrame::ColorSelectionFrame( QWidget* parent ):
    QWidget( parent ),
    Counter("ColorSelectionFrame" ),
    locked_( false )
{
    Debug::Throw( "ColorSelectionFrame::ColorSelectionFrame.\n" );

    _installActions();

    QVBoxLayout* mainLayout = new QVBoxLayout();
    mainLayout->setMargin(0);
    mainLayout->setSpacing( 5 );
    setLayout( mainLayout );

    QHBoxLayout* layout=new QHBoxLayout();
    layout->setMargin(0);
    layout->setSpacing( 10 );
    mainLayout->addLayout( layout );

    // color widget
    layout->addWidget( colorDisplay_ = new PRIVATE::ColorDisplay( this ), 0, Qt::AlignHCenter|Qt::AlignTop );

    // display grid
    GridLayout* gridLayout = new GridLayout();
    gridLayout->setMargin(0);
    gridLayout->setSpacing( 2 );
    gridLayout->setMaxCount(4);
    gridLayout->setColumnAlignment( 0, Qt::AlignRight|Qt::AlignVCenter );
    gridLayout->setColumnAlignment( 2, Qt::AlignRight|Qt::AlignVCenter );
    layout->addLayout( gridLayout );

    gridLayout->addWidget( new QLabel( "Name: ", this ) );

    QHBoxLayout *hLayout = new QHBoxLayout();
    hLayout->setSpacing(2);
    hLayout->setMargin(0);
    gridLayout->addLayout( hLayout, 0, 1, 1, 3 );

    // editor
    hLayout->addWidget( editor_ = new AnimatedLineEditor( this ) );
    editor_->setAlignment( Qt::AlignCenter );
    connect( editor_, SIGNAL( textEdited( const QString& ) ), SLOT( _fromEditor( const QString& ) ) );

    // save button
    QToolButton *button = new QToolButton( this );
    button->setDefaultAction( saveAction_ );
    button->setAutoRaise( true );

    button->setIcon( IconEngine::get( ICONS::SAVE ) );
    button->setToolTip( "Save current color to list" );
    hLayout->addWidget( button, 0 );

    // spinboxes
    SpinBox *spinbox;
    gridLayout->addWidget( new QLabel( "Red:", this ) );
    gridLayout->addWidget( spinbox = new SpinBox( this, RED ) );
    spinboxes_.insert( RED,  spinbox );

    gridLayout->addWidget( new QLabel( "Hue:", this ) );
    gridLayout->addWidget( spinbox = new SpinBox( this, HUE ) );
    spinboxes_.insert( HUE, spinbox );

    gridLayout->addWidget( new QLabel( "Green:", this ) );
    gridLayout->addWidget( spinbox = new SpinBox( this, GREEN ) );
    spinboxes_.insert( GREEN, spinbox );

    gridLayout->addWidget( new QLabel( "Saturation:", this ) );
    gridLayout->addWidget( spinbox = new SpinBox( this, SATURATION ) );
    spinboxes_.insert( SATURATION,  spinbox );

    gridLayout->addWidget( new QLabel( "Blue:", this ) );
    gridLayout->addWidget( spinbox = new SpinBox( this, BLUE ) );
    spinboxes_.insert( BLUE, spinbox );

    gridLayout->addWidget( new QLabel( "Value:", this ) );
    gridLayout->addWidget( spinbox = new SpinBox( this, VALUE ) );
    spinboxes_.insert( VALUE, spinbox );

    gridLayout->addWidget( new QLabel( "Alpha:", this ) );
    gridLayout->addWidget( spinbox = new SpinBox( this, ALPHA ) );
    spinboxes_.insert( ALPHA,  spinbox );

    gridLayout->setColumnStretch( 1, 1 );
    gridLayout->setColumnStretch( 3, 1 );

    for( SpinBoxMap::iterator iter = spinboxes_.begin(); iter!=spinboxes_.end(); ++iter )
    {
        iter.value()->setMinimum(0);
        iter.value()->setMaximum(255);
        iter.value()->setValue(0);
    }

    spinboxes_[ALPHA]->setValue(255);

    // color selection widget
    mainLayout->addWidget( colorWidget_ = new PRIVATE::ColorWidget( this ), 0, Qt::AlignCenter );
    connect( colorWidget_, SIGNAL( colorSelected( QString ) ), SLOT( setColor( QString ) ) );

    // connections
    connect( spinboxes_[HUE], SIGNAL( valueChanged( int ) ), this, SLOT( _fromHsv() ) );
    connect( spinboxes_[SATURATION], SIGNAL( valueChanged( int ) ), this, SLOT( _fromHsv() ) );
    connect( spinboxes_[VALUE], SIGNAL( valueChanged( int ) ), this, SLOT( _fromHsv() ) );

    connect( spinboxes_[RED], SIGNAL( valueChanged( int ) ), this, SLOT( _fromRgb() ) );
    connect( spinboxes_[GREEN], SIGNAL( valueChanged( int ) ), this, SLOT( _fromRgb() ) );
    connect( spinboxes_[BLUE], SIGNAL( valueChanged( int ) ), this, SLOT( _fromRgb() ) );
    connect( spinboxes_[ALPHA], SIGNAL( valueChanged( int ) ), this, SLOT( _fromRgb() ) );

    // transition widget
    transitionWidget_ = new TransitionWidget( this );
    transitionWidget_->hide();
    connect( &transitionWidget_->timeLine(), SIGNAL( finished() ), transitionWidget_, SLOT( hide() ) );

    return;
}

//_________________________________________________________
QAbstractButton& ColorSelectionFrame::colorWidget( void ) const
{ return *colorWidget_; }

//_________________________________________________________
void ColorSelectionFrame::setColor( QString value )
{
    Debug::Throw() << "ColorSelectionFrame::setColor - " << value << endl;
    QColor color( value );
    if( color.isValid() ) setColor( color.rgba() );
}

//_________________________________________________________
void ColorSelectionFrame::setColor( const QRgb& rgb )
{
    Debug::Throw() << "ColorSelectionFrame::setColor - " << rgb << endl;

    // check color
    QColor color( rgb );
    if( !color.isValid() )
    {
        InformationDialog( this, "selected color is not valid" ).exec();
        return;
    }

    // restore focus to the text editor
    /*
    this is needed because focus was lost when clicking in the list.
    this must be done before animation (if any) started, to avoid
    flicker
    */
    editor_->setFocus();
    if( transitionWidget_->isEnabled() && isVisible() )
    { transitionWidget_->initialize(); }

    // update color
    color_ = color;
    Debug::Throw() << "ColorSelectionFrame::setColor - color set" << endl;

    spinboxes_[RED]->setValue( qRed( rgb ) );
    spinboxes_[GREEN]->setValue( qGreen( rgb ) );
    spinboxes_[BLUE]->setValue( qBlue( rgb ) );

    int hue( 0 ), saturation( 0 ), value( 0 );
    color.getHsv( &hue, &saturation, &value );
    spinboxes_[HUE]->setValue( hue );
    spinboxes_[SATURATION]->setValue( saturation );
    spinboxes_[VALUE]->setValue( value );

    spinboxes_[ALPHA]->setValue( color.alpha() );

    // update label
    _updateLabel();

    if( transitionWidget_->isEnabled() && isVisible() )
    { transitionWidget_->start(); }

}


//_________________________________________________________
void ColorSelectionFrame::_fromEditor( const QString& text )
{
    Debug::Throw( "ColorSelectionFrame::_fromEditor.\n" );

    if( locked_ ) return;
    locked_ = true;

    // retrieve RGB indexes
    QColor color( text );
    if( !color.isValid() )
    {
        locked_ = false;
        return;
    }

    int hue( 0 ), saturation( 0 ), value( 0 );
    color.getHsv( &hue, &saturation, &value );

    spinboxes_[HUE]->setValue(hue);
    spinboxes_[SATURATION]->setValue(saturation);
    spinboxes_[VALUE]->setValue(value);

    int red( color.red() );
    int green( color.green() );
    int blue( color.blue() );

    spinboxes_[RED]->setValue(red);
    spinboxes_[GREEN]->setValue(green);
    spinboxes_[BLUE]->setValue(blue);

    // update color and display
    color_ = color;
    colorDisplay_->setColor( color_ );
    colorDisplay_->update();

    locked_ = false;
}

//_________________________________________________________
void ColorSelectionFrame::_fromRgb( void )
{
    Debug::Throw( "ColorSelectionFrame::_fromRgb.\n" );

    if( locked_ ) return;
    locked_ = true;

    // retrieve RGB indexes
    int red( spinboxes_[RED]->value() );
    int green( spinboxes_[GREEN]->value() );
    int blue( spinboxes_[BLUE]->value() );
    int alpha( spinboxes_[ALPHA]->value() );

    QColor color;
    color.setRgba( qRgba( red, green, blue, alpha ) );
    if( !color.isValid() )
    {
        locked_ = false;
        return;
    }

    int hue( 0 ), saturation( 0 ), value( 0 );
    color.getHsv( &hue, &saturation, &value );

    spinboxes_[HUE]->setValue(hue);
    spinboxes_[SATURATION]->setValue(saturation);
    spinboxes_[VALUE]->setValue(value);

    // update color
    color_ = color;

    // update label
    _updateLabel();
    locked_ = false;

}

//_________________________________________________________
void ColorSelectionFrame::_fromHsv( void )
{
    Debug::Throw( "ColorSelectionFrame::_fromHsv.\n" );

    if( locked_ ) return;
    locked_ = true;

    // retrieve RGB indexes
    int hue( spinboxes_[HUE]->value() );
    int saturation( spinboxes_[SATURATION]->value() );
    int value( spinboxes_[VALUE]->value() );

    QColor color;
    color.setHsv( hue, saturation, value );
    if( !color.isValid() ) {
        locked_ = false;
        return;
    }

    int red( 0 ), green( 0 ), blue( 0 );
    color.getRgb( &red, &green, &blue );

    spinboxes_[RED]->setValue(red);
    spinboxes_[GREEN]->setValue(green);
    spinboxes_[BLUE]->setValue(blue);

    // update color
    color_ = color;

    // update label
    _updateLabel();
    locked_ = false;
}

//_________________________________________________________
void ColorSelectionFrame::_updateLabel( void )
{

    Debug::Throw( "ColorSelectionFrame::_updateLabel.\n" );

    // update color widget
    colorDisplay_->setColor( color_ );
    colorDisplay_->update();

    // update label text
    editor_->setText( color_.name() );


}


//_________________________________________________________________________
void ColorSelectionFrame::_installActions( void )
{
    Debug::Throw( "ColorSelectionFrame::_installActions.\n" );

    // save
    addAction( saveAction_ = new QAction( IconEngine::get( ICONS::SAVE ), "&Save", this ) );
    saveAction_->setToolTip( "Add current color to list" );

}
